@@ -690,3 +690,211 @@ class MOPITT_CO_NASA_HDF_V5_filetype():
                 'Information Content Index' : [],
                 'Signal Chi2' : [],
                 'Swath Index ' : ['3']}
+
+class HDFmodisl2_generic_filetype():
+   parser = "HDFmodisl2"
+   doutf = "MODIS_simp_avg_netCDF"
+   parserParms = {}
+
+class MODIS_AOD_NASA_Collection_5_filetype():
+   parser = "HDFmodisl2"
+   doutf = "MODIS_simp_avg_netCDF"
+   parserParms = {}
+   time = "Scan_Start_Time"
+   longitude = "Longitude"
+   latitude = "Latitude"
+   outUnits = {"Scan_Start_Time": 'seconds',
+               "Solar_Zenith": 'degrees',
+               "Solar_Azimuth": 'degrees',
+               "Sensor_Zenith": 'degrees',
+               "Sensor_Azimuth": 'degrees',
+               "Scattering_Angle": 'degrees',
+               "Optical_Depth_Land_And_Ocean": 'Unitless',
+               "Image_Optical_Depth_Land_And_Ocean": 'Unitless',
+               "Optical_Depth_Ratio_Small_Land_And_Ocean": 'Unitless',
+               "Mean_Reflectance_Land_All": 'Unitless',
+               "Standard_Deviation_Reflectance_Land_All": 'Unitless',
+               "Path_Radiance_Land": 'Unitless',
+               "Error_Path_Radiance_Land": 'Unitless',
+               "Critical_Reflectance_Land": 'Unitless',
+               "Error_Critical_Reflectance_Land": 'Unitless',
+               "QualityWeight_Path_Radiance_Land": 'Unitless',
+               "QualityWeight_Critical_Reflectance_Land": 'Unitless',
+               "Aerosol_Type_Land": 'Unitless',
+               "Fitting_Error_Land": 'Unitless',
+               "Surface_Reflectance_Land": 'Unitless',
+               "Corrected_Optical_Depth_Land": 'Unitless',
+               "Corrected_Optical_Depth_Land_wav2p1": 'Unitless',
+               "Optical_Depth_Small_Land": 'Unitless',
+               "Optical_Depth_Ratio_Small_Land": 'Unitless',
+               "Number_Pixels_Used_Land": 'Pixels',
+               "Mean_Reflectance_Land": 'Unitless',
+               "STD_Reflectance_Land": 'Unitless',
+               "Mass_Concentration_Land": 'g/cm^2 x 1.0e-6',
+               "Angstrom_Exponent_Land": 'Unitless',
+               "Cloud_Fraction_Land": 'Unitless',
+               "Aerosol_Cldmask_Byproducts_Land": 'Unitless',
+               "Solution_Index_Ocean_Small": 'Unitless',
+               "Solution_Index_Ocean_Large": 'Unitless',
+               "Effective_Optical_Depth_Best_Ocean": 'Unitless',
+               "Effective_Optical_Depth_Average_Ocean": 'Unitless',
+               "Optical_Depth_Small_Best_Ocean": 'Unitless',
+               "Optical_Depth_Small_Average_Ocean": 'Unitless',
+               "Optical_Depth_Large_Best_Ocean": 'Unitless',
+               "Optical_Depth_Large_Average_Ocean": 'Unitless',
+               "Mass_Concentration_Ocean": 'g/cm^2 x 1.0e-6',
+               "Effective_Radius_Ocean": 'm x 1.0e^-6',
+               "Cloud_Condensation_Nuclei_Ocean": 'CCN/cm^2 x 1.0e10',
+               "Asymmetry_Factor_Best_Ocean": 'Unitless',
+               "Asymmetry_Factor_Average_Ocean": 'Unitless',
+               "Backscattering_Ratio_Best_Ocean": 'Unitless',
+               "Backscattering_Ratio_Average_Ocean": 'Unitless',
+               "Angstrom_Exponent_1_Ocean": 'Unitless',
+               "Angstrom_Exponent_2_Ocean": 'Unitless',
+               "Least_Squares_Error_Ocean": 'Unitless',
+               "Optical_Depth_Ratio_Small_Ocean_0.55micron": 'Unitless',
+               "Optical_Depth_by_models_ocean": 'Unitless',
+               "Cloud_Fraction_Ocean": 'Unitless',
+               "Number_Pixels_Used_Ocean": 'Pixels',
+               "Mean_Reflectance_Ocean": 'Unitless',
+               "STD_Reflectance_Ocean": 'Unitless',
+               "Aerosol_Cldmask_Byproducts_Ocean": 'Unitless',
+               "Deep_Blue_Aerosol_Optical_Depth_550_Land": 'Unitless',
+               "Deep_Blue_Aerosol_Optical_Depth_Land": 'Unitless',
+               "Deep_Blue_Angstrom_Exponent_Land": 'Unitless',
+               "Deep_Blue_Single_Scattering_Albedo_Land": 'Unitless',
+               "Deep_Blue_Surface_Reflectance_Land": 'Unitless',
+               "Deep_Blue_Mean_Reflectance_Land": 'Unitless',
+               "Deep_Blue_Number_Pixels_Used_Land": 'Pixels',
+               "Deep_Blue_Aerosol_Optical_Depth_550_Land_STD": 'Unitless',
+               "Deep_Blue_Aerosol_Optical_Depth_Land_STD": 'Unitless'}
+   extraDimLabels = {"Scan_Start_Time": [],
+                "Solar_Zenith": [],
+                "Solar_Azimuth": [],
+                "Sensor_Zenith": [],
+                "Sensor_Azimuth": [],
+                "Scattering_Angle": [],
+                "Optical_Depth_Land_And_Ocean": [],
+                "Image_Optical_Depth_Land_And_Ocean": [],
+                "Optical_Depth_Ratio_Small_Land_And_Ocean": [],
+                "Mean_Reflectance_Land_All": ['Land Solution 3 (470,659,2130nm)'],
+                "Standard_Deviation_Reflectance_Land_All": ['Land Solution 3 (470,659,2130nm)'],
+                "Path_Radiance_Land": ['Land Solution 1 (470,659nm)'],
+                "Error_Path_Radiance_Land": ['Land Solution 1 (470,659nm)'],
+                "Critical_Reflectance_Land": ['Land Solution 1 (470,659nm)'],
+                "Error_Critical_Reflectance_Land": ['Land Solution 1 (470,659nm)'],
+                "QualityWeight_Path_Radiance_Land": ['Land Solution 1 (470,659nm)'],
+                "QualityWeight_Critical_Reflectance_Land": ['Land Solution 1 (470,659nm)'],
+                "Aerosol_Type_Land": [],
+                "Fitting_Error_Land": [],
+                "Surface_Reflectance_Land": ['Land Solution 2 (470,555,659nm)'],
+                "Corrected_Optical_Depth_Land": ['Land Solution 3 (470,659,2130nm)'],
+                "Corrected_Optical_Depth_Land_wav2p1": [],
+                "Optical_Depth_Small_Land": ['Land Solution 4 (0.47,0.55,0.66,2.13um)'],
+                "Optical_Depth_Ratio_Small_Land": [],
+                "Number_Pixels_Used_Land": ['Land Solution 1 (470,659nm)'],
+                "Mean_Reflectance_Land": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "STD_Reflectance_Land": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Mass_Concentration_Land": [],
+                "Angstrom_Exponent_Land": [],
+                "Cloud_Fraction_Land": [],
+                "Aerosol_Cldmask_Byproducts_Land": ['Num_By_Products'],
+                "Solution_Index_Ocean_Small": ['Ocean Solution (best,average)'],
+                "Solution_Index_Ocean_Large": ['Ocean Solution (best,average)'],
+                "Effective_Optical_Depth_Best_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Effective_Optical_Depth_Average_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Optical_Depth_Small_Best_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Optical_Depth_Small_Average_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Optical_Depth_Large_Best_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Optical_Depth_Large_Average_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Mass_Concentration_Ocean": ['Ocean Solution (best,average)'],
+                "Effective_Radius_Ocean": ['Ocean Solution (best,average)'],
+                "Cloud_Condensation_Nuclei_Ocean": ['Ocean Solution (best,average)'],
+                "Asymmetry_Factor_Best_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Asymmetry_Factor_Average_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Backscattering_Ratio_Best_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Backscattering_Ratio_Average_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Angstrom_Exponent_1_Ocean": ['Ocean Solution (best,average)'],
+                "Angstrom_Exponent_2_Ocean": ['Ocean Solution (best,average)'],
+                "Least_Squares_Error_Ocean": ['Ocean Solution (best,average)'],
+                "Optical_Depth_Ratio_Small_Ocean_0.55micron": ['Ocean Solution (best,average)'],
+                "Optical_Depth_by_models_ocean": ['Solution Index'],
+                "Cloud_Fraction_Ocean": [],
+                "Number_Pixels_Used_Ocean": [],
+                "Mean_Reflectance_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "STD_Reflectance_Ocean": ['MODIS Band (470,555,659,865,1240,1640,2130nm)'],
+                "Aerosol_Cldmask_Byproducts_Ocean": ['Num_By_Products'],
+                "Deep_Blue_Aerosol_Optical_Depth_550_Land": [],
+                "Deep_Blue_Aerosol_Optical_Depth_Land": ['Num_DeepBlue_Wavelengths'],
+                "Deep_Blue_Angstrom_Exponent_Land": [],
+                "Deep_Blue_Single_Scattering_Albedo_Land": ['Num_DeepBlue_Wavelengths'],
+                "Deep_Blue_Surface_Reflectance_Land": ['Num_DeepBlue_Wavelengths'],
+                "Deep_Blue_Mean_Reflectance_Land": ['Num_DeepBlue_Wavelengths'],
+                "Deep_Blue_Number_Pixels_Used_Land": ['Num_DeepBlue_Wavelengths'],
+                "Deep_Blue_Aerosol_Optical_Depth_550_Land_STD": [],
+                "Deep_Blue_Aerosol_Optical_Depth_Land_STD": ['Num_DeepBlue_Wavelengths']}
+   extraDimSize = {"Scan_Start_Time": [],
+               "Solar_Zenith": [],
+               "Solar_Azimuth": [],
+               "Sensor_Zenith": [],
+               "Sensor_Azimuth": [],
+               "Scattering_Angle": [],
+               "Optical_Depth_Land_And_Ocean": [],
+               "Image_Optical_Depth_Land_And_Ocean": [],
+               "Optical_Depth_Ratio_Small_Land_And_Ocean": [],
+               "Mean_Reflectance_Land_All": ['3'],
+               "Standard_Deviation_Reflectance_Land_All": ['3'],
+               "Path_Radiance_Land": ['2'],
+               "Error_Path_Radiance_Land": ['2'],
+               "Critical_Reflectance_Land": ['2'],
+               "Error_Critical_Reflectance_Land": ['2'],
+               "QualityWeight_Path_Radiance_Land": ['2'],
+               "QualityWeight_Critical_Reflectance_Land": ['2'],
+               "Aerosol_Type_Land": [],
+               "Fitting_Error_Land": [],
+               "Surface_Reflectance_Land": ['3'],
+               "Corrected_Optical_Depth_Land": ['3'],
+               "Corrected_Optical_Depth_Land_wav2p1": [],
+               "Optical_Depth_Small_Land": ['4'],
+               "Optical_Depth_Ratio_Small_Land": [],
+               "Number_Pixels_Used_Land": ['2'],
+               "Mean_Reflectance_Land": ['7'],
+               "STD_Reflectance_Land": ['7'],
+               "Mass_Concentration_Land": [],
+               "Angstrom_Exponent_Land": [],
+               "Cloud_Fraction_Land": [],
+               "Aerosol_Cldmask_Byproducts_Land": ['7'],
+               "Solution_Index_Ocean_Small": ['2'],
+               "Solution_Index_Ocean_Large": ['2'],
+               "Effective_Optical_Depth_Best_Ocean": ['7'],
+               "Effective_Optical_Depth_Average_Ocean": ['7'],
+               "Optical_Depth_Small_Best_Ocean": ['7'],
+               "Optical_Depth_Small_Average_Ocean": ['7'],
+               "Optical_Depth_Large_Best_Ocean": ['7'],
+               "Optical_Depth_Large_Average_Ocean": ['7'],
+               "Mass_Concentration_Ocean": ['2'],
+               "Effective_Radius_Ocean": ['2'],
+               "Cloud_Condensation_Nuclei_Ocean": ['2'],
+               "Asymmetry_Factor_Best_Ocean": ['7'],
+               "Asymmetry_Factor_Average_Ocean": ['7'],
+               "Backscattering_Ratio_Best_Ocean": ['7'],
+               "Backscattering_Ratio_Average_Ocean": ['7'],
+               "Angstrom_Exponent_1_Ocean": ['2'],
+               "Angstrom_Exponent_2_Ocean": ['2'],
+               "Least_Squares_Error_Ocean": ['2'],
+               "Optical_Depth_Ratio_Small_Ocean_0.55micron": ['2'],
+               "Optical_Depth_by_models_ocean": ['9'],
+               "Cloud_Fraction_Ocean": [],
+               "Number_Pixels_Used_Ocean": [],
+               "Mean_Reflectance_Ocean": ['7'],
+               "STD_Reflectance_Ocean": ['7'],
+               "Aerosol_Cldmask_Byproducts_Ocean": ['7'],
+               "Deep_Blue_Aerosol_Optical_Depth_550_Land": [],
+               "Deep_Blue_Aerosol_Optical_Depth_Land": ['3'],
+               "Deep_Blue_Angstrom_Exponent_Land": [],
+               "Deep_Blue_Single_Scattering_Albedo_Land": ['3'],
+               "Deep_Blue_Surface_Reflectance_Land": ['3'],
+               "Deep_Blue_Mean_Reflectance_Land": ['3'],
+               "Deep_Blue_Number_Pixels_Used_Land": ['3'],
+               "Deep_Blue_Aerosol_Optical_Depth_550_Land_STD": [],
+               "Deep_Blue_Aerosol_Optical_Depth_Land_STD": ['3']}
