@@ -45,7 +45,7 @@ def global_intersect_map_geo(parser, griddef, verbose=True):
     '''
     For each pixel, find all gridcells that it intersects
 
-    This function does not compute or save teh fractional 
+    This function does not compute or save the fractional 
     overlap of individual pixels.  
 
     This function is designed to handle grids that span 
@@ -58,6 +58,8 @@ def global_intersect_map_geo(parser, griddef, verbose=True):
     automatically wrap pixels that fall off the east/west edges
     of the projection.
 
+    Pixels with NaN for any vertex are rejected
+
     Assumptions:
         - Straight lines in projected space adequately 
         approximate the edges of pixels/gridcells.
@@ -69,14 +71,15 @@ def global_intersect_map_geo(parser, griddef, verbose=True):
     '''
     if verbose:
         print('Mapping '+parser.name+'\nat '+str(datetime.datetime.now()))  
+    outer_indices = griddef.indLims()
     # create the dictionary we'll use as a map
-    map = map_helpers.init_output_map(griddef.indLims())
+    map = map_helpers.init_output_map(outer_indices)
     map['parser'] = parser
         # we're going to hold onto both the prepared and unprepared versions
     # of the polys, so we can access the fully method set in the unprep
     # polys, but still do fast comparisons
-    gridPolys = map_helpers.rect_grid_polys(griddef.indLims())
-    prepPolys = map_helpers.rect_grid_polys(griddef.indLims())
+    gridPolys = map_helpers.rect_grid_polys(outer_indices)
+    prepPolys = map_helpers.rect_grid_polys(outer_indices)
     if verbose: print('prepping polys in grid')
     for poly in prepPolys.itervalues():
         poly = prep(poly)  # prepare these, they're going to get compared a lot
@@ -96,6 +99,9 @@ def global_intersect_map_geo(parser, griddef, verbose=True):
     maxCol = griddef.indLims()[3] + 1
     midCol = (minCol+maxCol)/2.0
     for (pxrow, pxcol, pxind) in izip(row, col, ind):
+        if (numpy.any(numpy.isnan(pxrow)) or 
+            numpy.any(numpy.isnan(pxcol))):
+            continue # skip incomplete pixels
         pointsTup = zip(pxrow, pxcol)
         prelimPoly = geom.MultiPoint(pointsTup).convex_hull
         (bbBot, bbLeft, bbTop, bbRight) = prelimPoly.bounds
@@ -116,7 +122,7 @@ def global_intersect_map_geo(parser, griddef, verbose=True):
             pixPolys = [prelimPoly]
         # try intersecting the poly(s) with all the grid polygons
         for poly in pixPolys:
-            for key in gridPolys.iterkeys():
+            for key in map_helpers.get_possible_cells(outer_indices, poly):
                 if prepPolys[key].intersects(poly) and not gridPolys[key].touches(poly):
                     map[key].append((tuple(pxind), None))
     if verbose: print('Done intersecting.')
@@ -137,6 +143,8 @@ def regional_intersect_map_geo(parser, griddef, verbose=True):
     relies on the assumption that any pixel of interest 
     will have at least one corner within the bounds of 
     the grid
+
+    Pixels with NaN for any vertex are rejected
     
     Several assumptions are made:
         - Straight lines in projected space adequately 
@@ -149,14 +157,15 @@ def regional_intersect_map_geo(parser, griddef, verbose=True):
     
     if verbose:
         print('Mapping '+parser.name+'\nat '+str(datetime.datetime.now()))
-    map = map_helpers.init_output_map(griddef.indLims())
+    outer_indices = griddef.indLims()
+    map = map_helpers.init_output_map(outer_indices)
     map['parser'] = parser
-    bounds = prep(map_helpers.rect_bound_poly(griddef.indLims()))
+    bounds = prep(map_helpers.rect_bound_poly(outer_indices))
     # we're going to hold onto both the prepared and unprepared versions
     # of the polys, so we can access the fully method set in the unprep
     # polys, but still do fast comparisons
-    gridPolys = map_helpers.rect_grid_polys(griddef.indLims())
-    prepPolys = map_helpers.rect_grid_polys(griddef.indLims())
+    gridPolys = map_helpers.rect_grid_polys(outer_indices)
+    prepPolys = map_helpers.rect_grid_polys(outer_indices)
     if verbose: print('prepping polys in grid')
     for poly in prepPolys.itervalues():
         poly = prep(poly)  # prepare these, they're going to get compared a lot
@@ -175,7 +184,9 @@ def regional_intersect_map_geo(parser, griddef, verbose=True):
         sys.stdout.write("Approximately 0 pixels gridded. ")
         sys.stdout.flush()
         for (pxrow, pxcol, pxind) in izip(row, col, ind):
-            if not any([bounds.contains(geom.asPoint((r,c))) \
+            if numpy.any(numpy.isnan(pxrow)) or numpy.any(numpy.isnan(pxcol)):
+                continue  # if we have only a partial pixel, skip
+            elif not any([bounds.contains(geom.asPoint((r,c))) \
                        for (r,c) in izip(pxrow, pxcol)]):
                 continue  # if none of the corners are in bounds, skip
             griddedPix += 1
@@ -184,18 +195,20 @@ def regional_intersect_map_geo(parser, griddef, verbose=True):
             sys.stdout.flush()
             pixPoly = geom.MultiPoint(zip(pxrow, pxcol)).convex_hull
             
-            for key in gridPolys.iterkeys():
+            for key in map_helpers.get_possible_cells(outer_indices, pixPoly):
                 if prepPolys[key].intersects(pixPoly) and not \
                                   gridPolys[key].touches(pixPoly) :
                     map[key].append((tuple(pxind), None))
         print('Done intersecting.')
     else:
         for (pxrow, pxcol, pxind) in izip(row, col, ind):
-            if not any([bounds.contains(geom.asPoint((r,c))) for (r,c) \
+            if numpy.any(numpy.isnan(pxrow)) or numpy.any(numpy.isnan(pxcol)):
+                continue  # if we have only a partial pixel, skip
+            elif not any([bounds.contains(geom.asPoint((r,c))) for (r,c) \
                                         in izip(pxrow, pxcol)]):
                 continue  # if none of the corners are in bounds, skip
             pixPoly = geom.MultiPoint(zip(pxrow, pxcol)).convex_hull
-            for key in gridPolys.iterkeys():
+            for key in map_helpers.get_possible_cells(outer_indices, pixPoly):
                 if prepPolys[key].intersects(pixPoly) and not \
                                   gridPolys[key].touches(pixPoly):
                     map[key].append((tuple(pxind), None))
