@@ -76,6 +76,15 @@ class invalidPixCeption(Exception):
 tai93conv = lambda(timestring):utils.timestr_to_nsecs(timestring, 
                                '00:00:00_01-01-1993', '%H:%M:%S_%m-%d-%Y')
 
+def boolCaster(boolStr):
+    if boolStr == 'True':
+        return True
+    elif boolStr == 'False':
+        return False
+    else:
+        msg = 'Attempt to cast invalid string %s to boolean' % boolStr
+        raise TypeError(msg)
+
 # currently borked.  No immediate plans to fix
 #class OMNO2e_wght_avg_out_func(out_func):
 class OMNO2e_wght_avg_BORKED(out_func): 
@@ -323,6 +332,10 @@ class OMNO2e_netCDF_avg_out_func(out_func):
             The value we want to use to denote missing data
             in the output file.  This will be documented
             within the output file itself.
+        includePixelCount:
+            If this parameter is True, WHIPS will include a field
+            'ValidPixelCount' in the output file that will include
+            the number of valid pixels for each grid cell.
 
     Outputs a netcdf file with name determined by outFileName
     parameter.  This netcdf file contains as many variables
@@ -338,7 +351,7 @@ class OMNO2e_netCDF_avg_out_func(out_func):
                 'outUnits', 'extraDimLabel', 'extraDimSize',
                 'timeComparison', 'timeStart', 'timeStop',
                 'cloudFractUpperCutoff', 'solarZenAngUpperCutoff',
-                'pixIndXtrackAxis', 'fillVal']
+                'pixIndXtrackAxis', 'fillVal', 'includePixelCount']
     @staticmethod
     def required_parms():
         return {'overallQualFlag' : ('The name of the field containing ' \
@@ -422,7 +435,12 @@ class OMNO2e_netCDF_avg_out_func(out_func):
                                       'parser, check this).','int'),
                 'fillVal' : ('The value to use as a fill value in the output ' \
                              'netCDF file.  This value will replace any missing '\
-                             'or invalid output values','decimal')}
+                             'or invalid output values','decimal'), 
+                'includePixelCount' : ('If set to true, the output will include '\
+                                       'a field "ValidPixelCount" that contains '\
+                                       'the number of valid pixels in each grid '\
+                                       'cell.  Only pixels with nonzero weight  '\
+                                       'are considered valid.', 'bool')}
     # variable signifying which list is to act as the master list index
     __userKeys__ = "inFieldNames"
 
@@ -451,7 +469,7 @@ class OMNO2e_netCDF_avg_out_func(out_func):
                     'timeComparison':str, 'timeStart':tai93conv,
                     'timeStop':tai93conv, 'cloudFractUpperCutoff':float,
                     'solarZenAngUpperCutoff':int, 'pixIndXtrackAxis':int,
-                    'fillVal':float}
+                    'fillVal':float, 'includePixelCount':boolCaster}
         for (k,func) in castDict.items():
             try:
                 self.parmDict[k] = func(self.parmDict[k])
@@ -476,6 +494,7 @@ class OMNO2e_netCDF_avg_out_func(out_func):
         (minRow, maxRow, minCol, maxCol) = griddef.indLims()
         nRows = maxRow - minRow + 1
         nCols = maxCol - minCol + 1
+        nValidPixels = numpy.zeros((nRows, nCols))
         sumWght = numpy.zeros((nRows, nCols, 1))  # needs extra dim to generalize for 3D vars
         sumVars = dict()
         for field, size in zip(self.parmDict['inFieldNames'], self.parmDict['extraDimSize']):
@@ -495,58 +514,77 @@ class OMNO2e_netCDF_avg_out_func(out_func):
                 if verbose:
                     print('Processing {0} for output at {1}.'.format(\
                             parser.name, str(datetime.datetime.now())))
-                # loop over gridboxes in map and calculate weights
-                for (gridCell, pixTup) in map.iteritems():
-                    # translate gridCell to account for possible non-zero ll corner
-                    gridRow = gridCell[0]
-                    gridCol = gridCell[1]
-                    gridInd = (gridRow - minRow, gridCol - minCol)
-                    # get the values needed to calculate weight
-                    for (pxInd, unused_weight) in pixTup:
-                        # check summary flag
-                        sumFlag = parser.get_cm(self.parmDict['overallQualFlag'], pxInd)
-                        if sumFlag % 2:
-                            continue
-                        # check cloud fraction flag
-                        cFrac = parser.get_cm(self.parmDict['cloudFrac'], pxInd)
-                        if not (cFrac <= self.parmDict['cloudFractUpperCutoff']):
-                            continue
-                        # check solar zenith angle flag
-                        solZenAng = parser.get_cm(self.parmDict['solarZenithAngle'], pxInd)
-                        if solZenAng > self.parmDict['solarZenAngUpperCutoff']:
-                            continue
-                        # check time flag
-                        time = parser.get_cm(self.parmDict['time'], pxInd)
-                        # calculate and factor in offset if the user wanted us to
-                        if self.parmDict['timeComparison'] == 'local':
-                            pixLon = parser.get_cm(self.parmDict['longitude'], pxInd)
-                            offset = utils.UTCoffset_from_lon(pixLon)
-                            time += offset
-                        if time < self.parmDict['timeStart'] or time > self.parmDict['timeStop']:
-                            continue
-                        # read in all the data, abandon ship if data is all NaN
-                        rawDataDict = {}
-                        try:
-                            for field in self.parmDict['inFieldNames']:
+
+                for field in self.parmDict['inFieldNames']:
+                    # loop over gridboxes in map and calculate weights
+                    for (gridCell, pixTup) in map.iteritems():
+                        # translate gridCell to account 
+                        # for possible non-zero ll corner
+                        gridRow = gridCell[0]
+                        gridCol = gridCell[1]
+                        gridInd = (gridRow - minRow, gridCol - minCol)
+                        # get the values needed to calculate weight
+                        for (pxInd, unused_weight) in pixTup:
+                            # check summary flag
+                            sumFlag = parser.get_cm(\
+                                      self.parmDict['overallQualFlag'], pxInd)
+                            if sumFlag % 2:
+                                continue
+                            # check cloud fraction flag
+                            cFrac = parser.get_cm(\
+                                    self.parmDict['cloudFrac'], pxInd)
+                            if not (cFrac <= self.parmDict[\
+                                                  'cloudFractUpperCutoff']):
+                                continue
+                            # check solar zenith angle flag
+                            solZenAng = parser.get_cm(\
+                                       self.parmDict['solarZenithAngle'], pxInd)
+                            if solZenAng > self.parmDict[\
+                                                  'solarZenAngUpperCutoff']:
+                                continue
+                            # check time flag
+                            time = parser.get_cm(self.parmDict['time'], pxInd)
+                            # calculate and factor in offset 
+                            # if the user wanted us to
+                            if self.parmDict['timeComparison'] == 'local':
+                                pixLon = parser.get_cm(\
+                                             self.parmDict['longitude'], pxInd)
+                                offset = utils.UTCoffset_from_lon(pixLon)
+                                time += offset
+                            if time < self.parmDict['timeStart'] \
+                                   or time > self.parmDict['timeStop']:
+                                continue
+                            # read in all the data 
+                            # abandon ship if data is all NaN
+                            rawDataDict = {}
+                            try:
+                                #==========================
                                 rawData = parser.get_cm(field, pxInd)
+                                #==========================
                                 if numpy.isnan(rawData).all():
                                     raise invalidPixCeption
                                 rawDataDict[field] = rawData
-                        except invalidPixCeption:
-                            continue
-                        # compute the weight
-                        fov = pxInd[self.parmDict['pixIndXtrackAxis']]
-                        weight = _OMNO2e_formula(cFrac, fov)
-                        assert weight != numpy.NaN
-                        # add the weight tot the total for this cell
-                        sumWght[gridInd] += weight
-                        for field in self.parmDict['inFieldNames']:
+                            except invalidPixCeption:
+                                continue
+                            # compute the weight
+                            fov = pxInd[self.parmDict['pixIndXtrackAxis']]
+                            weight = _OMNO2e_formula(cFrac, fov)
+                            assert weight != numpy.NaN
+                            if weight > 0:
+                                nValidPixels[gridInd] += 1
+
+                            # add the weight to the total for this cell 
+                            sumWght[gridInd] += weight
                             weightVals = rawDataDict[field] * weight
                             if weightVals.size > 1:
-                                sumVars[field][gridInd] = numpy.nansum([sumVars[field][gridInd], weightVals], axis=0)        
+                                sumVars[field][gridInd] = \
+                                     numpy.nansum([sumVars[field][gridInd],\
+                                                   weightVals], axis=0)        
                             else:
-                                sumVars[field][gridInd] = numpy.nansum([sumVars[field][gridInd][0], weightVals])
-                map['parser'] = parser  # return parser to map
+                                sumVars[field][gridInd] = \
+                                     numpy.nansum([sumVars[field][gridInd][0],\
+                                                   weightVals])
+            map['parser'] = parser  # return parser to map
                 
         # divide out variables by weights to get avgs. 
         oldSettings = numpy.seterr(divide='ignore')
@@ -607,11 +645,19 @@ class OMNO2e_netCDF_avg_out_func(out_func):
             varHandle[:] = avgs[field]
             # assign variable attributes
             setattr(varHandle, 'Units', units[field])
+        # Write out the pixel counts if the user requested them
+        if self.parmDict['includePixelCount']:
+            varDims = ('row', 'col')
+            varHandle = outFid.createVariable('ValidPixelCount', 'i', varDims, 
+                                              fill_value=self.parmDict['fillVal'])
+            varHandle[:] = nValidPixels
         outFid.close()
-        # create a dict with teh same data as avgs, but diff names
+        # create a dict with the same data as avgs, but diff names
         outAvg = dict()
         for (k,v) in avgs.iteritems():
             outAvg[outFnames[k]] = v
+        if self.parmDict['includePixelCount']:
+            outAvg['ValidPixelCount'] = nValidPixels
         return outAvg
     
 class wght_avg_netCDF(out_func):
@@ -809,17 +855,9 @@ class wght_avg_netCDF(out_func):
         self.parmDict['dimSizes'] = sizeIntTups
 
         # process logNormal
-        def boolCaster(boolStr):
-            if boolStr == 'True':
-                return True
-            elif boolStr == 'False':
-                return False
-            else:
-                msg = 'Attempt to cast invalid string %s to boolean' % boolStr
-                raise ValueError(msg)
         try:
             self.parmDict['logNormal'] = [boolCaster(el) for el in self.parmDict['logNormal']]
-        except ValueError:
+        except TypeError:
             print('Bad string in logNormal.  Must be either "True" or "False". Exiting.')
             raise
 
@@ -1325,3 +1363,326 @@ class unweighted_filtered_MOPITT_avg_netCDF_out_func(wght_avg_netCDF):
 
         # invoke parent's constructor
         wght_avg_netCDF.__init__(self, parmDict)
+
+
+
+class MODIS_simp_avg_netCDF_out_func(wght_avg_netCDF):
+
+   @staticmethod
+   def parm_list():
+      # return ordered list of required parameters
+      return ['timeStart', 'timeStop', 'timeComparison', 'time', 'fillVal', 'inFieldNames', 'outUnits', 'outFieldNames', 'extraDimSize', 'extraDimLabels']
+
+   @staticmethod
+   def required_parms():
+      # return dictionary containing parm:('description',type) entries for each parameter
+      return {'timeStart':('The earliest time for which data should be '\
+                           'recorded into the output file.  All times '\
+                           'before this time in the input file(s) will be '\
+                           'filtered out.  Must be in the format: hh:mm:ss_'\
+                           'MM-DD-YYYY','time'),
+              'timeStop':('The latest time for which data should be recorded '\
+                          'into the output file.  All times after this time '\
+                          'in the input file(s) will be filtered out.   '\
+                          'Must be in the format: hh:mm:ss_MM-DD-YYYY', 'time'),
+              'timeComparison':('Must be set to either "local" or "UTC".  '\
+                                'Determines how the file timestamps are '\
+                                'compared to the start/stop time.  If set '\
+                                'to "local", then the file timestamps are '\
+                                'converted to local time on a pixel-by-pixel '\
+                                'basis (using longitude to estimate time zone)'\
+                                ' before being compared to time boundaries.  '\
+                                'If set to "UTC" the file timestamps '\
+                                '(which are assumed to be in UTC) are compared'\
+                                ' against the start/stop time directly.', None),
+              'time':('The name of the field containing timestamps.  '\
+                      'Timestamps are assumed to be in the '\
+                      'TAI-93 format.', None),
+              'fillVal':('The value to use as a fill value in the output '\
+                         'netCDF file.  This value will replace any '\
+                         'missing or invalid output values', 'decimal'),
+              'inFieldNames':('The names of the fields desired to be output.  '\
+                              'Input as a comma-delimited list.', 'list'),
+              'outFieldNames':('The names of the output variables. (even if '\
+                               'they are to be the same as input variables).  '\
+                               'Should be a comma-delimited list co-indexed '\
+                               'to inFieldNames', 'list'),
+              'outUnits':('The units of the variables to be written out.' \
+                          '  Should be a comma-delimited list co-indexed '\
+                          'to inFieldNames', 'list'),
+              'extraDimSize':('List of the sizes of the extra dimensions in '\
+                              'the output file.  Must be a comma-delimited '\
+                              'list of integers.  Fields with no extra '\
+                              'dimensions should be left blank.  For example, '\
+                              'if there are four inFields, the first and '\
+                              'third of which have no extra dimensions, the '\
+                              'second of which has one (which has length '\
+                              'four), and the fourth has one (which has '\
+                              'length five), the extraDimSize entry should '\
+                              'look like this: ,4,,5 '\
+                              'The list must be co-indexed to inFieldNames', 
+                              'listoflists'),
+              'extraDimLabels':('List of names of the extra dimensions in the '\
+                                'output file.  Must be a comma-delimited list '\
+                                'of strings.  Fields with no extra dimensions '\
+                                'should be left blank.  '\
+                                'For example, if there are four inFields, '\
+                                'the first and third of which have no extra '\
+                                'dimensions, the second of which has one '\
+                                '("foo"), and the fourth has one ("bar"), the '\
+                                'extraDimLabels entry should look like this: '\
+                                ',foo,,bar '\
+                                'The list must be co-indexed to inFieldNames',
+                                'listoflists'),
+               'includePixelCount':('If set to true, the output will include '\
+                                    'a field "ValidPixelCount" that contains '\
+                                    'the number of valid pixels in each grid '\
+                                    'cell.  Only pixels with nonzero weight  '\
+                                    'are considered valid.','bool')}
+
+   # variable signifying which list is to act as the master list index
+   __userKeys__ = 'inFieldNames'
+   def __init__(self, pDict):
+      '''Convert input to format of parent input'''
+      # make a shallow copy to the parameter dict, as we'll be making changes
+      # and we don't want to mutate the argument
+      self.parmDict = dict(pDict)
+
+      # even though IO interface handles casting already
+      # a catchblock has been added here for safety
+      # in case someone wants to use this class directly
+      castDict = {'time':str, 'inFieldNames':list,
+                  'outUnits':list, 'outFieldNames':list,
+                  'extraDimLabels':list, 'extraDimSize':list, 
+                  'timeStart':tai93conv, 'timeStop':tai93conv,
+                  'timeComparison':str, 'fillVal':float, 
+                  'includePixelCount':boolCaster}
+      for (k,func) in castDict.items():
+         try:
+            self.parmDict[k] = func(self.parmDict[k])
+         except TypeError:
+            pass
+   
+      # by this point times are already converted to TAI93 standard
+      # no need to convert here
+      self.parmDict['timeConv'] = lambda(x):x
+
+      # create Weighting function
+      def wghtFunc(parser, index, prevWght):
+         '''
+         There is no weighting function-- NASA has already weighted pixels
+         based on their overall quality.  Only the highest quality pixels
+         have been included for land-based data, and medium to high quality
+         pixels have been weighted appropriately for ocean-based data.
+         '''
+         return 1
+      self.parmDict['weightFunction'] = wghtFunc
+  
+      # create filtering function
+      def filterFunc(parser, indStack):
+         '''
+         A filtering function is also not necessary for MODIS data.
+         No pixels were removed by this filter.
+         '''
+         return numpy.zeros(indStack.shape)
+ 
+
+   def __call__(self, maps, griddef, outfilename, verbose, version):
+      '''Write out an averaged file in netcdf format.'''
+
+      # Run sanity checks on values (e.g. logical start/stop times) 
+      # raise IOerror on failure
+      ###
+      if self.parmDict['timeStart'] > self.parmDict['timeStop']:
+         msg = 'Input start time must come before stop time.'
+         raise IOError(msg)
+      if (len(self.parmDict['inFieldNames']) !=  \
+              len(self.parmDict['outFieldNames']) or  
+              len(self.parmDict['inFieldNames']) !=  \
+              len(self.parmDict['outUnits']) or      
+              len(self.parmDict['inFieldNames']) !=  \
+              len(self.parmDict['extraDimLabels'])):
+         msg = 'All field/unit inputs ' + \
+               'should have the same number of elements.'
+         raise IOError(msg)
+
+      # Create numpy arrays to hold data
+      ###
+      (minRow, maxRow, minCol, maxCol) = griddef.indLims()
+      nRows = maxRow - minRow + 1
+      nCols = maxCol - minCol + 1
+      sumWght = numpy.zeros((nRows, nCols, 1)) # needs extra dim to generalize for 3D vars
+      sumVars = dict()
+      for field, size in zip(self.parmDict['inFieldNames'], self.parmDict['extraDimSize']):
+         if size:
+            sumVars[field] = numpy.zeros((nRows, nCols, int(size[0])))
+         else:
+            # pad with a singlet dim if it was 2D
+            sumVars[field] = numpy.zeros((nRows, nCols, 1))
+      nValidPixels = numpy.zeros((nRows, nCols))
+      nNaNPixels = numpy.zeros((nRows, nCols))
+
+      # loop over maps
+      ###
+      if not isinstance(maps, list):
+         maps = [maps] # create list if we only got a single map
+      for map in maps:
+         pixSum = 0
+         totofNaN = 0
+         # open up context manager
+         with map.pop('parser') as parser: # remove parser for looping
+            if verbose:
+               print('\nProcessing {0} for output at {1}.'.format(\
+                      parser.name, str(datetime.datetime.now())))
+               sys.stdout.write('\nData added from 0 pixels. ')
+               sys.stdout.flush()
+            # loop over gridboxes in map and calculate weights
+            for (gridCell, pixTup) in map.iteritems():
+               # translate gridCell to account for possible non-zero ll corner
+               gridRow = gridCell[0]
+               gridCol = gridCell[1]
+               gridInd = (gridRow - minRow, gridCol - minCol)
+               # get the values needed to calculate weight
+               for (pxInd, unused_weight) in pixTup:
+
+
+                  # check quality flags 
+                  # and continue if pixel fails to meet requirements
+                  # ***To be added as an option in a future update***
+                  ###
+                  '''
+                  QAL_flag = parser.get_cm('Quality_Assurance_Land', pxInd)
+                  if QAL_flag % 2 != 1:
+                     QAO_flag = parser.get_cm('Quality_Assurance_Ocean', pxInd)
+                     if QAO_flag % 2 != 1:
+                        continue
+                  '''
+
+                  # check time within start/stop range
+                  ###
+                  time = parser.get_cm(self.parmDict['time'], pxInd)
+                  if self.parmDict['timeComparison'] == 'local':
+                     pixLon = parser.get_cm(self.parmDict['longitude'], pxInd)
+                     offset = utils.UTCoffset_from_lon(pixLon)
+                     time += offset
+                  if time < self.parmDict['timeStart']\
+                                   or time > self.parmDict['timeStop']:
+                     continue
+                                                      
+                  # read in all the data, abandon ship if data is all NaN
+                  ###
+                  rawDataDict = {}
+                  try:
+                     for field in self.parmDict['inFieldNames']:
+                        rawData = parser.get_cm(field, pxInd)
+                        rawDataDict[field] = rawData
+                     if all(numpy.isnan(el).all()\
+                                    for el in rawDataDict.values()):
+                        raise invalidPixCeption
+                  except invalidPixCeption:
+                     nNaNPixels[gridInd] += 1
+                     totofNaN += 1
+                     continue              
+                  # compute the weight and add to the total for this cell
+                  ###
+                  pixSum = pixSum + 1
+                  if verbose:
+                      sys.stdout.write("\rData added from {0} pixels. NaNs found for {1} pixels."\
+                                                         .format(pixSum, totofNaN))
+                      sys.stdout.flush()
+                  sumWght[gridInd] += 1
+                  nValidPixels[gridInd] += 1
+                  for field in self.parmDict['inFieldNames']:
+                     if rawDataDict[field].size > 1:
+                        sumVars[field][gridInd] = numpy.nansum([sumVars[field][gridInd], rawDataDict[field]], axis=0)
+                     else:
+                        sumVars[field][gridInd] = numpy.nansum([sumVars[field][gridInd][0], rawDataDict[field]])
+            map['parser'] = parser # return parser to map
+
+      # divide out variables by weights to get avgs
+      ###
+      oldSettings = numpy.seterr(divide='ignore')
+      avgs = dict()
+      with warnings.catch_warnings():
+         warnings.simplefilter("ignore")
+         for (field,var) in sumVars.iteritems():
+            unfiltAvgs = var/sumWght
+            filtAvgs = numpy.where(sumWght != 0, unfiltAvgs, self.parmDict['fillVal'])
+            # strip trailing singlet for 2D arrays
+            if filtAvgs.shape[-1] == 1:
+               avgs[field] = filtAvgs.reshape(filtAvgs.shape[0:2])
+            else:
+               avgs[field] = filtAvgs
+      numpy.seterr(divide=oldSettings['divide'])
+     
+      if verbose:
+          print "\nWriting final output to file at {0}.".format(\
+                                  str(datetime.datetime.now()))
+      # associate coindexed parameters into dicts keyed on inFieldnames
+      # so we can loop by field
+      ###
+      outFnames = dict(izip(self.parmDict['inFieldNames'], self.parmDict['outFieldNames']))
+      units = dict(izip(self.parmDict['inFieldNames'], self.parmDict['outUnits']))
+      extraDim = dict(izip(self.parmDict['inFieldNames'], self.parmDict['extraDimLabels']))
+
+      # write out results to a netcdf file
+      ###
+      outFid = netCDF4.Dataset(outfilename, 'w', format='NETCDF3_CLASSIC')
+
+      # create the 2 dimensions all files use
+      ###
+      outFid.createDimension('row', nRows)
+      outFid.createDimension('col', nCols)
+
+      # write global attributes
+      ###
+      setattr(outFid, 'Version', vsnmsg(version))
+      # File_start_time, File_end_time, requirements for valid data, 
+      # time comparison scheme, etc. written in same manner
+      setattr(outFid, 'Input_files', ' '.join([map['parser'].name for map in maps]))
+      setattr(outFid, 'Projection', griddef.__class__.__name__[:-8])
+
+      for(k,v) in griddef.parms.iteritems():
+         setattr(outFid, k, v)
+      # loop over fields and write variables
+      ###
+      for field in self.parmDict['inFieldNames']:
+         # create tuple of dimensions, defining new dim if necessary
+         if len(avgs[field].shape) == 2:
+            # only row/cols
+            varDims = ('row', 'col')
+         elif len(avgs[field].shape) == 3:
+            # has extra dim
+            dimName = extraDim[field][0]
+            dimSize = avgs[field].shape[2]
+            if dimName not in outFid.dimensions.keys():
+               outFid.createDimension(dimName, dimSize)
+            varDims = ('row', 'col', dimName)
+  
+         # create and assign value to variable
+         ###
+         varHandle = outFid.createVariable(outFnames[field], 'd', varDims, \
+                                           fill_value=self.parmDict['fillVal'])
+         varHandle[:] = avgs[field]
+    
+         # assign variable attributes
+         ###
+         setattr(varHandle, 'Units', units[field])
+
+      if self.parmDict['includePixelCount']:
+         varDims = ('row', 'col')
+         varHandle = outFid.createVariable('ValidPixelCount', 'i', varDims, 
+                                           fill_value=self.parmDict['fillVal'])
+         varHandle[:] = nValidPixels
+         varHandle = outFid.createVariable('NaNPixelCount', 'i', varDims,
+                                           fill_value=self.parmDict['fillVal'])
+         varHandle[:] = nNaNPixels
+
+      outFid.close()
+      # create a dict with the same data as avgs, but diff names
+      ###
+      outAvg = dict()
+      for (k,v) in avgs.iteritems():
+         outAvg[outFnames[k]] = v
+      return outAvg
+
