@@ -283,7 +283,7 @@ class HDF4File(GeoFile):
             fid.close
         
         # convert missing values if appropriate
-        if missingValue and vData.dtype in ['float32', 'float64']:
+        if missingValue and vData.dtype in ['float32', 'float64', 'int16']:
             vData = numpy.where(vData == missingValue, numpy.NaN, vData)
 
         # use indices if we have them
@@ -781,13 +781,16 @@ class HDFmopittl2_File(HDF4File):
     """
     Provide interface to MOPITT level 2 V5 product
 
-    Automatically setes the missing value for the data to
+    Automatically sets the missing value for the data to
     -9999.0, as this is the missing value used (but not 
     documented) within the data.
     """
-    _nameExpMap = {'Time' : '/MOP02/Geolocation Fields/Time',
-                   'Latitude' : '/MOP02/Geolocation Fields/Latitude', 
-                   'Longitude' : '/MOP02/Geolocation Fields/Longitude',
+    __GeoPath = '/MOP02/Geolocation Fields/'
+    __DataPath = '/MOP02/Data Fields/'
+
+    _nameExpMap = {'Time' : __GeoPath + 'Time',
+                   'Latitude' : __GeoPath + 'Latitude', 
+                   'Longitude' : __GeoPath + 'Longitude',
                    'Seconds in Day' : '/MOP02/Data Fields/Seconds in Day',
                    'Pressure Grid' : '/MOP02/Data Fields/Pressure Grid',
                    'Solar Zenith Angle' : '/MOP02/Data Fields/Solar Zenith Angle',
@@ -836,3 +839,329 @@ class HDFmopittl2_File(HDF4File):
         struct = numpy.zeros(lat.size, dtype = protoDtype)
         (struct['lat'], struct['lon'], struct['ind']) = (lat, lon, ind)
         return struct
+
+
+class HDFmodisl2_File(HDF4File):
+    """
+    Provide interface to MODIS level 2 V5 product
+    
+    Uses the missing values provided in the _FVMap
+    """
+    __GeoPath = '/mod04/Geolocation Fields/'
+    __DataPath = '/mod04/Data Fields/'
+
+    _nameExpMap = {'Latitude' : __GeoPath+'Latitude', 
+                   'Longitude' : __GeoPath+'Longitude',
+                   'Scan_Start_Time' : __DataPath+'Scan_Start_Time',
+                   'Solar_Zenith' : __DataPath+'Solar_Zenith',
+                   'Solar_Azimuth' : __DataPath+'Solar_Azimuth',
+                   'Sensor_Zenith' : __DataPath+'Sensor_Zenith',
+                   'Sensor_Azimuth' : __DataPath+'Sensor_Azimuth',
+                   'Cloud_Mask_QA' : __DataPath+'Cloud_Mask_QA',
+                   'Scattering_Angle' : __DataPath+'Scattering_Angle',
+                   'Optical_Depth_Land_And_Ocean' : __DataPath + \
+                                 'Optical_Depth_Land_And_Ocean',
+                   'Image_Optical_Depth_Land_And_Ocean' : __DataPath + \
+                                 'Image_Optical_Depth_Land_And_Ocean',
+                   'Optical_Depth_Ratio_Small_Land_And_Ocean' : __DataPath + \
+                                 'Optical_Depth_Ratio_Small_Land_And_Ocean',
+                   'Mean_Reflectance_Land_All' : __DataPath + \
+                                 'Mean_Reflectance_Land_All',
+                   'Standard_Deviation_Reflectance_Land_All' : __DataPath + \
+                                 'Standard_Deviation_Reflectance_Land_All',
+                   'Path_Radiance_Land' : __DataPath+'Path_Radiance_Land',
+                   'Error_Path_Radiance_Land' : __DataPath + \
+                                 'Error_Path_Radiance_Land',
+                   'Critical_Reflectance_Land' : __DataPath + \
+                                 'Critical_Reflectance_Land',
+                   'Error_Critical_Reflectance_Land' : __DataPath + \
+                                 'Error_Critical_Reflectance_Land',
+                   'QualityWeight_Path_Radiance_Land' : __DataPath + \
+                                 'QualityWeight_Path_Radiance_Land',
+                   'QualityWeight_Critical_Reflectance_Land' : __DataPath + \
+                                 'QualityWeight_Critical_Reflectance_Land',
+                   'Aerosol_Type_Land' : __DataPath+'Aerosol_Type_Land',
+                   'Fitting_Error_Land' : __DataPath+'Fitting_Error_Land',
+                   'Surface_Reflectance_Land' : __DataPath + \
+                                 'Surface_Reflectance_Land',
+                   'Corrected_Optical_Depth_Land' : __DataPath + \
+                                 'Corrected_Optical_Depth_Land',
+                   'Corrected_Optical_Depth_Land_wav2p1' : __DataPath + \
+                                 'Corrected_Optical_Depth_Land_wav2p1',
+                   'Optical_Depth_Small_Land' : __DataPath + \
+                                 'Optical_Depth_Small_Land',
+                   'Optical_Depth_Ratio_Small_Land' : __DataPath + \
+                                 'Optical_Depth_Ratio_Small_Land',
+                   'Number_Pixels_Used_Land' : __DataPath + \
+                                 'Number_Pixels_Used_Land',
+                   'Mean_Reflectance_Land' : __DataPath+'Mean_Reflectance_Land',
+                   'STD_Reflectance_Land' : __DataPath+'STD_Reflectance_Land',
+                   'Mass_Concentration_Land' : __DataPath + \
+                                 'Mass_Concentration_Land',
+                   'Angstrom_Exponent_Land' : __DataPath + \
+                                 'Anstrom_Exponent_Land',
+                   'Cloud_Fraction_Land' : __DataPath+'Cloud_Fraction_Land',
+                   'Quality_Assurance_Land' : __DataPath + \
+                                 'Quality_Assurance_Land',
+                   'Quality_Assurance_Crit_Ref_Land' : __DataPath + \
+                                 'Quality_Assurance_Crit_Ref_Land',
+                   'Aerosol_Cldmask_Byproducts_Land' : __DataPath + \
+                                 'Aerosol_Cldmask_Byproducts_Land',
+                   'Solution_Index_Ocean_Small' : __DataPath + \
+                                 'Solution_Index_Ocean_Small',
+                   'Solution_Index_Ocean_Large' : __DataPath + \
+                                 'Solution_Index_Ocean_Large',
+                   'Effective_Optical_Depth_Best_Ocean' : __DataPath + \
+                                 'Effective_Optical_Depth_Best_Ocean',
+                   'Effective_Optical_Depth_Average_Ocean' : __DataPath + \
+                                 'Effective_Optical_Depth_Average_Ocean',
+                   'Optical_Depth_Small_Best_Ocean' : __DataPath + \
+                                 'Optical_Depth_Small_Best_Ocean',
+                   'Optical_Depth_Small_Average_Ocean' : __DataPath + \
+                                 'Optical_Depth_Small_Average_Ocean',
+                   'Optical_Depth_Large_Best_Ocean' : __DataPath + \
+                                 'Optical_Depth_Large_Best_Ocean',
+                   'Optical_Depth_Large_Average_Ocean' : __DataPath + \
+                                 'Optical_Depth_Large_Average_Ocean',
+                   'Mass_Concentration_Ocean' : __DataPath + \
+                                 'Mass_Concentration_Ocean',
+                   'Effective_Radius_Ocean' : __DataPath + \
+                                 'Effective_Radius_Ocean',
+                   'Cloud_Condensation_Nuclei_Ocean' : __DataPath + \
+                                 'Cloud_Condensation_Nuclei_Ocean',
+                   'Asymmetry_Factor_Best_Ocean' : __DataPath + \
+                                 'Asymmetry_Factor_Best_Ocean',
+                   'Asymmetry_Factor_Average_Ocean' : __DataPath + \
+                                 'Assymetry_Factor_Average_Ocean',
+                   'Backscattering_Ratio_Best_Ocean' : __DataPath + \
+                                 'Backscattering_Ratio_Best_Ocean',
+                   'Backscattering_Ratio_Average_Ocean' : __DataPath + \
+                                 'Backscattering_Ratio_Average_Ocean',
+                   'Angstrom_Exponent_1_Ocean' : __DataPath + \
+                                 'Angstrom_Exponent_1_Ocean',
+                   'Angstrom_Exponent_2_Ocean' : __DataPath + \
+                                 'Angstrom_Exponent_2_Ocean',
+                   'Least_Squares_Error_Ocean' : __DataPath + \
+                                 'Least_Squares_Error_Ocean',
+                   'Optical_Depth_Ratio_Small_Ocean_0.55micron' : __DataPath + \
+                                 'Optical_Depth_Ratio_Small_Ocean_0.55micron',
+                   'Optical_Depth_by_models_ocean' : __DataPath + \
+                                 'Optical_Depth_by_models_ocean',
+                   'Cloud_Fraction_Ocean' : __DataPath+'Cloud_Fraction_Ocean',
+                   'Number_Pixels_Used_Ocean' : __DataPath + \
+                                 'Number_Pixels_Used_Ocean',
+                   'Mean_Reflectance_Ocean' : __DataPath + \
+                                 'Mean_Reflectance_Ocean',
+                   'STD_Reflectance_Ocean' : __DataPath + \
+                                 'STD_Reflectance_Ocean',
+                   'Quality_Assurance_Ocean' : __DataPath + \
+                                 'Quality_Assurance_Ocean',
+                   'Aerosol_Cldmask_Byproducts_Ocean' : __DataPath + \
+                                 'Aerosol_Cldmask_Byproducts_Ocean',
+                   'Deep_Blue_Aerosol_Optical_Depth_550_Land' : __DataPath + \
+                                 'Deep_Blue_Aerosol_Optical_Depth_550_Land',
+                   'Deep_Blue_Aerosol_Optical_Depth_Land' : __DataPath + \
+                                 'Deep_Blue_Optical_Depth_Land',
+                   'Deep_Blue_Angstrom_Exponent_Land' : __DataPath + \
+                                 'Deep_Blue_Angstrom_Exponent_Land',
+                   'Deep_Blue_Single_Scattering_Albedo_Land' : __DataPath + \
+                                 'Deep_blue_Single_Scattering_Albedo_Land',
+                   'Deep_Blue_Surface_Reflectance_Land' : __DataPath + \
+                                 'Deep_Blue_Surface_Reflectance_Land',
+                   'Deep_Blue_Mean_Reflectance_Land' : __DataPath + \
+                                 'Deep_Blue_Mean_Reflectance_Land',
+                   'Deep_Blue_Number_Pixels_Used_Land' : __DataPath + \
+                                 'Deep_Blue_Number_Pixels_Used_Land',
+                   'Deep_Blue_Aerosol_Optical_Depth_550_Land_STD' : __DataPath+\
+                                 'Deep_Blue_Aerosol_Optical_Depth_550_Land_STD',
+                   'Deep_Blue_Aerosol_Optical_Depth_Land_STD' : __DataPath + \
+                                 'Deep_Blue_Aerosol_Optical_Depth_Land_Std'}
+
+    _indexMap = {'default' : lambda var, ind: var[..., ind[0], ind[1]],
+                 'Quality_Assurance_Land' : lambda var, ind: var[..., ind[0], ind[1], 0],
+                 'Quality_Assurance_Ocean' : lambda var, ind: var[..., ind[0], ind[1], 0]}
+
+    _FVMap = {'Latitude' : -999.0,
+              'Longitude' : -999.0,
+              'Scan_Start_Time' : -999.0,
+              'Solar_Zenith' : -9999,
+              'Solar_Azimuth' : -9999,
+              'Sensor_Zenith' : -9999,
+              'Sensor_Azimuth' : -9999,
+              'Cloud_Mask_QA' : 0,
+              'Scattering_Angle' : -9999,
+              'Optical_Depth_Land_And_Ocean' : -9999,
+              'Image_Optical_Depth_Land_And_Ocean' : -9999,
+              'Optical_Depth_Ratio_Small_Land_And_Ocean' : -9999,
+              'Mean_Reflectance_Land_All' : -9999,
+              'Standard_Deviation_Reflectance_Land_All' : -9999,
+              'Path_Radiance_Land' : -9999,
+              'Error_Path_Radiance_Land' : -9999,
+              'Critical_Reflectance_Land' : -9999,
+              'Error_Critical_Reflectance_Land' : -9999,
+              'QualityWeight_Path_Radiance_Land' : -9999,
+              'QualityWeight_Critical_Reflectance_Land' : -9999, 
+              'Aerosol_Type_Land' : -9999,
+              'Fitting_Error_Land' : -9999,
+              'Surface_Reflectance_Land' : -9999,
+              'Corrected_Optical_Depth_Land' : -9999,
+              'Corrected_Optical_Depth_Land_wav2p1' : -9999,
+              'Optical_Depth_Small_Land' : -9999,
+              'Optical_Depth_Ratio_Small_Land' : -9999,
+              'Number_Pixels_Used_Land' : -9999,
+              'Mean_Reflectance_Land' : -9999,
+              'STD_Reflectance_Land' : -9999,
+              'Mass_Concentration_Land' : -999.0,
+              'Angstrom_Exponent_Land' : -9999,
+              'Cloud_Fraction_Land' : -9999,
+              'Quality_Assurance_Land' : 0,
+              'Quality_Assurance_Crit_Ref_Land' : 0,
+              'Aerosol_Cldmask_Byproducts_Land' : -9999,
+              'Solution_Index_Ocean_Small' : -9999,
+              'Solution_Index_Ocean_Large' : -9999,
+              'Effective_Optical_Depth_Best_Ocean' : -9999,
+              'Effective_Optical_Depth_Average_Ocean' : -9999,
+              'Optical_Depth_Small_Best_Ocean' : -9999,
+              'Optical_Depth_Small_Average_Ocean' : -9999,
+              'Optical_Depth_Large_Best_Ocean' : -9999,
+              'Optical_Depth_Large_Average_Ocean' : -9999,
+              'Mass_Concentration_Ocean' : -999.0,
+              'Effective_Radius_Ocean' : -9999,
+              'Cloud_Condensation_Nuclei_Ocean' : -999.0,
+              'Asymmetry_Factor_Best_Ocean' : -9999,
+              'Asymmetry_Factor_Average_Ocean' : -9999,
+              'Backscattering_Ratio_Best_Ocean' : -9999,
+              'Backscattering_Ratio_Average_Ocean' : -9999,
+              'Angstrom_Exponent_1_Ocean' : -9999,
+              'Angstrom_Exponent_2_Ocean' : -9999,
+              'Least_Squares_Error_Ocean' : -9999,
+              'Optical_Depth_Ratio_Small_Ocean_0.55micron' : -9999,
+              'Optical_Depth_by_models_ocean' : -9999,
+              'Cloud_Fraction_Ocean' : -9999,
+              'Number_Pixels_Used_Ocean' : -9999,
+              'Mean_Reflectance_Ocean' : -9999,
+              'STD_Reflectance_Ocean' : -9999,
+              'Quality_Assurance_Ocean' : 0,
+              'Aerosol_Cldmask_Byproducts_Ocean' : -9999,
+              'Deep_Blue_Aerosol_Optical_Depth_550_Land' : -9999,
+              'Deep_Blue_Aerosol_Optical_Depth_Land' : -9999,
+              'Deep_Blue_Angstrom_Exponent_Land' : -9999,
+              'Deep_Blue_Single_Scattering_Albedo_Land' : -9999,
+              'Deep_Blue_Surface_Reflectance_Land' : -9999,
+              'Deep_Blue_Mean_Reflectance_Land' : -9999,
+              'Deep_Blue_Number_Pixels_Used_Land' : -9999,
+              'Deep_Blue_Aerosol_Optical_Depth_550_Land_STD' : -9999,
+              'Deep_Blue_Aerosol_Optical_Depth_Land_STD' : -9999}
+
+    _ScaleOffsetMap = {'Latitude' : None,
+                   'Longitude' : None,
+                   'Scan_Start_Time' : None,
+                   'Solar_Zenith' : (0.009999999776482582,0.0),
+                   'Solar_Azimuth' : (0.009999999776482582,0.0),
+                   'Sensor_Zenith' : (0.009999999776482582,0.0),
+                   'Sensor_Azimuth' : (0.009999999776482582,0.0),
+                   'Cloud_Mask_QA' : None,
+                   'Scattering_Angle' : (0.009999999776482582,0.0),
+                   'Optical_Depth_Land_And_Ocean' : (0.0010000000474974513,0.0),
+                   'Image_Optical_Depth_Land_And_Ocean' : (0.0010000000474974513,0.0),
+                   'Optical_Depth_Ratio_Small_Land_And_Ocean' : (0.0010000000474974513,0.0),
+                   'Mean_Reflectance_Land_All' : (0.00009999999747378752, 0.0),
+                   'Standard_Deviation_Reflectance_Land_All' : (0.00009999999747378752,0.0),
+                   'Path_Radiance_Land' : (0.00009999999747378752, 0.0),
+                   'Error_Path_Radiance_Land' : (0.0, 0.00009999999747378752),
+                   'Critical_Reflectance_Land' : (0.00009999999747378752, 0.0),
+                   'Error_Critical_Reflectance_Land' : (0.00009999999747378752, 0.0),
+                   'QualityWeight_Path_Radiance_Land' : None,
+                   'QualityWeight_Critical_Reflectance_Land' : None, 
+                   'Aerosol_Type_Land' : None,
+                   'Fitting_Error_Land' : (0.0010000000474974513, 0.0),
+                   'Surface_Reflectance_Land' : (0.0010000000474974513, 0.0),
+                   'Corrected_Optical_Depth_Land' : (0.0010000000474974513, 0.0),
+                   'Corrected_Optical_Depth_Land_wav2p1' : (0.0010000000474974513,0.0),
+                   'Optical_Depth_Small_Land' : (0.0010000000474974513,0.0),
+                   'Optical_Depth_Ratio_Small_Land' : (0.0010000000474974513, 0.0),
+                   'Number_Pixels_Used_Land' : None,
+                   'Mean_Reflectance_Land' : (0.00009999999747378752, 0.0),
+                   'STD_Reflectance_Land' : (0.00009999999747378752, 0.0),
+                   'Mass_Concentration_Land' : None,
+                   'Angstrom_Exponent_Land' : (0.0010000000474974513, 0.0),
+                   'Cloud_Fraction_Land' : (0.0010000000474974513, 0.0),
+                   'Quality_Assurance_Land' : None,
+                   'Quality_Assurance_Crit_Ref_Land' : None,
+                   'Aerosol_Cldmask_Byproducts_Land' : (0.0010000000474974513, 0.0),
+                   'Solution_Index_Ocean_Small' : None,
+                   'Solution_Index_Ocean_Large' : None,
+                   'Effective_Optical_Depth_Best_Ocean' : (0.0010000000474974513, 0.0),
+                   'Effective_Optical_Depth_Average_Ocean' : (0.0010000000474974513, 0.0),
+                   'Optical_Depth_Small_Best_Ocean' : (0.0010000000474974513, 0.0),
+                   'Optical_Depth_Small_Average_Ocean' : (0.0010000000474974513, 0.0),
+                   'Optical_Depth_Large_Best_Ocean' : (0.0010000000474974513, 0.0),
+                   'Optical_Depth_Large_Average_Ocean' : (0.0010000000474974513, 0.0),
+                   'Mass_Concentration_Ocean' : None,
+                   'Effective_Radius_Ocean' : (0.0010000000474974513, 0.0),
+                   'Cloud_Condensation_Nuclei_Ocean' : None,
+                   'Asymmetry_Factor_Best_Ocean' : (0.0010000000474974513, 0.0),
+                   'Asymmetry_Factor_Average_Ocean' : (0.0010000000474974513, 0.0),
+                   'Backscattering_Ratio_Best_Ocean' : (0.0010000000474974513, 0.0),
+                   'Backscattering_Ratio_Average_Ocean' : (0.0010000000474974513, 0.0),
+                   'Angstrom_Exponent_1_Ocean' : (0.0010000000474974513, 0.0),
+                   'Angstrom_Exponent_2_Ocean' : (0.0010000000474974513, 0.0),
+                   'Least_Squares_Error_Ocean' : (0.0010000000474974513, 0.0),
+                   'Optical_Depth_Ratio_Small_Ocean_0.55micron' : (0.0010000000474974513, 0.0),
+                   'Optical_Depth_by_models_ocean' : (0.0010000000474974513, 0.0),
+                   'Cloud_Fraction_Ocean' : (0.0010000000474974513, 0.0),
+                   'Number_Pixels_Used_Ocean' : None,
+                   'Mean_Reflectance_Ocean' : (0.00009999999747378752, 0.0),
+                   'STD_Reflectance_Ocean' : (0.00009999999747378752, 0.0),
+                   'Quality_Assurance_Ocean' : None,
+                   'Aerosol_Cldmask_Byproducts_Ocean' : (0.0010000000474974513, 0.0),
+                   'Deep_Blue_Aerosol_Optical_Depth_550_Land' : (0.0010000000474974513, 0.0),
+                   'Deep_Blue_Aerosol_Optical_Depth_Land' :(0.0010000000474974513, 0.0),
+                   'Deep_Blue_Angstrom_Exponent_Land' : (0.0010000000474974513, 0.0),
+                   'Deep_Blue_Single_Scattering_Albedo_Land' : (0.0010000000474974513, 0.0),
+                   'Deep_Blue_Surface_Reflectance_Land' : (0.0010000000474974513, 0.0),
+                   'Deep_Blue_Mean_Reflectance_Land' : (0.00009999999747378752, 0.0),
+                   'Deep_Blue_Number_Pixels_Used_Land' : None,
+                   'Deep_Blue_Aerosol_Optical_Depth_550_Land_STD' : (0.0010000000474974513, 0.0),
+                   'Deep_Blue_Aerosol_Optical_Depth_Land_STD' : (0.0010000000474974513, 0.0)}
+
+    def get(self, key, indices=None):
+        '''Overloaded version of get that applies the correct missing value.'''
+        if self._ScaleOffsetMap[key]:
+            retval = HDF4File.get(self, key, indices, missingValue=self._FVMap[key])*self._ScaleOffsetMap[key][0]+self._ScaleOffsetMap[key][1]
+            return retval                                              
+        else:
+            retval = HDF4File.get(self, key, indices, missingValue=self._FVMap[key])
+            return retval
+
+    def get_cm(self, key, indices=None):
+        '''Overloaded version of get_cm that applied the correct missing value.'''
+        if self._ScaleOffsetMap[key]:
+           return HDF4File.get(self, key, indices, missingValue=self._FVMap[key])*self._ScaleOffsetMap[key][0]+self._ScaleOffsetMap[key][1]
+        else:
+           return HDF4File.get(self, key, indices, missingValue=self._FVMap[key])
+        
+    def get_geo_centers(self):
+        lat = self.get('Latitude').squeeze()
+        lon = self.get('Longitude').squeeze()
+        tem = numpy.arange(lat.size).reshape(lat.shape)
+        ind = numpy.transpose(numpy.array([tem/tem[0].size, tem%tem[0].size]),\
+                              (1,2,0))
+        protoDtype = [('lat', lat.dtype), ('lon', lon.dtype), \
+                              ('ind', ind.dtype, 2)]
+        struct = numpy.zeros(lat.shape, dtype=protoDtype)
+        (struct['lat'], struct['lon'], struct['ind']) = (lat, lon, ind)
+        return struct
+
+    def get_geo_corners(self):
+        '''Regional intersect cannot be used with MODIS data'''
+        raise NotImplementedError(\
+                                  'The regional_intersect and '\
+                                  'global_intersect\n                     '\
+                                  'mapping functions cannot be used with '\
+                                  'MODIS data,\n                     '\
+                                  'as no corner information is provided by '\
+                                  'this filetype.\n                     Please'\
+                                  ' use the point_in_cell mapping function '\
+                                  'instead.')
+
