@@ -7,7 +7,7 @@ Process a series of files, generating some kind of output for each
 If verbose is set to True, all default status updates will be printed.  
 If set to False, the program will run silently
 
-@version 3/1/2012
+@version 10/5/2012
 @author: maki, oberman
 '''
 import os
@@ -28,7 +28,7 @@ from process_sat import filetypes
 '''
 VERSION NUMBER
 '''
-__version__ = "1.1.0"
+__version__ = "1.1.6"
 
 class NeedToParseInFileException(Exception):
     '''exception class for signaling the need to parse input file'''
@@ -201,7 +201,7 @@ parser.add_argument('--outFileName', help='Optionally, supply the name ' \
                     'of the output file.  If no name is provided, the ' \
                     'output file will be named \'output1\'', metavar = \
                     'FileName')
-parser.add_argument('--includeGrid', nargs=1, metavar='GridFileName', \
+parser.add_argument('--includeGrid', metavar='GridFileName', \
                     help='Optionally, supply the name of a file to which to '\
                     'write out the latitudes and longitudes of the gridcells '\
                     'defined by the selected projection')
@@ -295,16 +295,6 @@ if not os.access(outDirectory, os.W_OK) or (os.path.isfile(outFileName)\
                         outDirectory, gnomespice.outFileName), 75)
     sys.exit(0)
 
-gridFileName = (gnomespice.includeGrid and \
-                        os.path.join(outDirectory,gnomespice.includeGrid[0]))
-if gridFileName and not os.access(gridFileName, os.W_OK):
-    print textwrap.wrap("Error: Unable to write output to file {1} in "\
-                        "directory {0}.  That directory may already "\
-                        "contain an existing file of that name, for which "\
-                        "you do not have write permissions.  Check the "\
-                        "output directory and try again.".format(\
-                        outDirectory, gnomespice.includeGrid), 75)
-
 # To be implemented later (maybe)
 parserList = None                        
 
@@ -606,9 +596,16 @@ for f in files:
 # Construct the grid definition
 if verbose: print('constructing grid '+str(datetime.datetime.now()))
 griddef = gridDef(gridDict)
+
+gridFileName = gnomespice.includeGrid
 if gridFileName:
-    if verbose: print('writing grid to file '+str(datetime.datetime.now()))
-    utils.write_grid_to_netcdf(griddef, gridFileName)
+    if not os.access(os.path.dirname(gridFileName), os.W_OK):
+        print textwrap.wrap("Warning: Unable to write output to file {0}.  "\
+                            "No gridcell file will be written for this run."\
+                            "".format(gridFileName), 75)
+    else:
+        if verbose: print('writing grid to file '+str(datetime.datetime.now()))
+        utils.write_grid_to_netcdf(griddef, gridFileName)
 
 # Map data to grid
 if verbose: print('calculating maps '+str(datetime.datetime.now()))
